@@ -1,6 +1,6 @@
 # kas - setup tool for bitbake based projects
 #
-# Copyright (c) Siemens AG, 2017-2018
+# Copyright (c) Siemens AG, 2017-2020
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -28,6 +28,7 @@ import logging
 import shutil
 import os
 import pprint
+import sys
 from .libkas import (ssh_cleanup_agent, ssh_setup_agent, ssh_no_host_key_check,
                      get_build_environ, repos_fetch, repos_apply_patches)
 from .includehandler import IncludeException
@@ -40,7 +41,40 @@ class Macro:
     """
         Contains commands and provides method to run them.
     """
-    def __init__(self):
+    def __init__(self, use_common_setup=True, use_common_cleanup=True):
+        if use_common_setup:
+            repo_loop = Loop('repo_setup_loop')
+            repo_loop.add(SetupReposStep())
+
+            self.setup_commands = [
+                SetupDir(),
+            ]
+
+            if ('SSH_PRIVATE_KEY' in os.environ
+                    or 'SSH_PRIVATE_KEY_FILE' in os.environ):
+                self.setup_commands.append(SetupSSHAgent())
+
+            self.setup_commands += [
+                SetupHome(),
+                InitSetupRepos(),
+                repo_loop,
+                FinishSetupRepos(),
+                ReposApplyPatches(),
+                SetupEnviron(),
+                WriteBBConfig(),
+            ]
+        else:
+            self.setup_commands = []
+
+        if (use_common_cleanup
+                and ('SSH_PRIVATE_KEY' in os.environ
+                     or 'SSH_PRIVATE_KEY_FILE' in os.environ)):
+            self.cleanup_commands = [
+                CleanupSSHAgent(),
+            ]
+        else:
+            self.cleanup_commands = []
+
         self.commands = []
 
     def add(self, command):
@@ -55,7 +89,9 @@ class Macro:
             configuration.
         """
         skip = skip or []
-        for command in self.commands:
+        joined_commands = self.setup_commands + \
+            self.commands + self.cleanup_commands
+        for command in joined_commands:
             command_name = str(command)
             if command_name in skip:
                 continue
@@ -112,6 +148,14 @@ class SetupHome(Command):
         Sets up the home directory of kas.
     """
 
+    # A list of environment variables that SETUP_HOME uses
+    # This should be kept up to date with any code in execute()
+    ENV_VARS = [
+        'GIT_CREDENTIAL_HELPER',
+        'AWS_CONFIG_FILE',
+        'AWS_SHARED_CREDENTIALS_FILE',
+    ]
+
     def __init__(self):
         super().__init__()
         self.tmpdirname = tempfile.mkdtemp()
@@ -131,9 +175,14 @@ class SetupHome(Command):
             fds.write('[User]\n'
                       '\temail = kas@example.com\n'
                       '\tname = Kas User\n')
+            if os.environ.get('GIT_CREDENTIAL_HELPER', False):
+                fds.write('[credential]\n'
+                          '\thelper = '
+                          + os.environ.get('GIT_CREDENTIAL_HELPER')
+                          + '\n')
 
-        if (os.environ.get('AWS_CONFIG_FILE', False)
-                and os.environ.get('AWS_SHARED_CREDENTIALS_FILE', False)):
+        if os.environ.get('AWS_CONFIG_FILE', False) \
+                and os.environ.get('AWS_SHARED_CREDENTIALS_FILE', False):
             os.makedirs(self.tmpdirname + "/.aws")
 
             shutil.copy(os.environ['AWS_CONFIG_FILE'],
@@ -192,7 +241,7 @@ class SetupEnviron(Command):
         return 'setup_environ'
 
     def execute(self, ctx):
-        ctx.environ.update(get_build_environ())
+        ctx.environ.update(get_build_environ(ctx.config.get_build_system()))
 
 
 class WriteBBConfig(Command):
@@ -215,6 +264,8 @@ class WriteBBConfig(Command):
                     sorted(layer for repo in ctx.config.get_repos()
                            for layer in repo.layers)))
                 fds.write('"\n')
+                fds.write('BBPATH ?= "${TOPDIR}"\n')
+                fds.write('BBFILES ??= ""\n')
 
         def _write_local_conf(ctx):
             filename = ctx.build_dir + '/conf/local.conf'
@@ -303,9 +354,12 @@ class SetupReposStep(Command):
 
         ctx.config.repo_dict = ctx.config._get_repo_dict()
 
-        ctx.missing_repos = [ctx.config.repo_dict[repo_name]
-                             for repo_name in ctx.missing_repo_names
-                             if repo_name in ctx.config.repo_dict]
+        ctx.missing_repos = []
+        for repo_name in ctx.missing_repo_names:
+            if repo_name not in ctx.config.repo_dict:
+                logging.error('Include references unknown repo: %s', repo_name)
+                sys.exit(1)
+            ctx.missing_repos.append(ctx.config.repo_dict[repo_name])
 
         repos_fetch(ctx.missing_repos)
 
@@ -335,12 +389,10 @@ class FinishSetupRepos(Command):
     def execute(self, ctx):
         """ TODO refactor protected-access """
         # now fetch everything with complete config and check out layers
-        # except if keep_config is set
-        if not ctx.keep_config:
-            repos_fetch(ctx.config.get_repos())
+        repos_fetch(ctx.config.get_repos())
 
-            for repo in ctx.config.get_repos():
-                repo.checkout()
+        for repo in ctx.config.get_repos():
+            repo.checkout()
 
         logging.debug('Configuration from config file:\n%s',
                       pprint.pformat(ctx.config._config))
